#!/usr/bin/env python
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import warnings
import sys
if sys.version_info < (3,):
    warnings.warn(
        "Python 2 reached the end of its life at the end of 2019.\n"
        "Please move to Python 3 because this module will drop support "
        "on {eod}.".format(
            eod='31-Jan-2021'
        ), DeprecationWarning)



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2020-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import yaml
import logging
import argparse

import opsramp.binding


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    parser.add_argument(
        'suffix',
        type=str
    )
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
    suffix = ns.suffix

    endpoint = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']

    ormp = opsramp.binding.connect(endpoint, key, secret)
    resp = ormp.get(suffix)
    print(yaml.dump(resp, default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
import requests_mock

import opsramp.binding


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.first_response = self.client.first_response()
        assert 'First_Response' in str(self.first_response)

    def test_search(self):
        group = self.first_response
        for pattern, expected in (
            ('', ['unit', 'test', 'results']),
            ('pageNo=1&pageSize=100&is&sortName=id', ['more', 'nonsense'])
        ):
            if pattern:
                url = group.api.compute_url('?' + pattern)
            else:
                url = group.api.compute_url()
            with requests_mock.Mocker() as m:
                m.get(url, json=expected, complete_qs=True)
                actual = group.search(pattern)
            assert actual == expected

    def test_policy_detail(self):
        group = self.first_response
        thisid = 789012
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.policy_detail(uuid=thisid)
            assert actual == expected

    def test_create(self):
        group = self.first_response
        expected = {'id': 345678}
        fake_defn = {'name': 'dougal'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=fake_defn)
            assert actual == expected

    def test_update(self):
        group = self.first_response
        thisid = 123456
        expected = {'id': thisid}
        fake_defn = {'name': 'ted'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_delete(self):
        group = self.first_response
        thisid = 789012
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected

    def test_enable(self):
        group = self.first_response
        thisid = 345678
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/enable' % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.enable(uuid=thisid)
            assert actual == expected

    def test_disable(self):
        group = self.first_response
        thisid = 901234
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/disable' % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.disable(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import logging
import argparse

import opsramp.binding


def connect():
    url = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ['OPSRAMP_TENANT_ID']

    ormp = connect()
    tenant = ormp.tenant(tenant_id)

    categs = tenant.rba().categories()
    clist = categs.get()
    # for each category
    for cdata in clist:
        print('category', cdata['id'], cdata['name'])
        thiscat = categs.category(cdata['id'])
        # for each script in this category
        for s in thiscat.get():
            print(
                '... script',
                s['id'],
                s['platforms'],
                s['executionType'],
                '"%s"' % s['name'],
                '"%s"' % s['description']
            )


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import logging
import argparse

import opsramp.binding
import opsramp.integrations


subscription = os.environ['AZURE_SUBSCRIPTION']
azure_tenant = os.environ['AZURE_TENANT_ID']
azure_client = os.environ['AZURE_CLIENT_ID']
azure_secret = os.environ['AZURE_SECRET_KEY']


def connect():
    url = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ['OPSRAMP_TENANT_ID']

    ormp = connect()
    tnt = ormp.tenant(tenant_id)
    discovery = tnt.discovery()

    # Create new discovery profile...
    creds = opsramp.Integrations.mkAzureARM(
        arm_subscription_id=subscription,
        arm_tenant_id=azure_tenant,
        arm_client_id=azure_client,
        arm_secret_key=azure_secret
    )
    jdata = [{
        'name': 'Azure ' + subscription,
        'credential': creds,
        'policy': {
            'name': 'whatever',
            'resourceType': 'ALL',
            'entityType': 'ALL',
            'rules': [{
                'filterType': 'ANY_CLOUD_RESOURCE'
            }],
            'actions': [{
                'action': 'MANAGE DEVICE',
                'items': [],
                'forceAssignOrUnassign': False
            }],
            'matchType': 'ANY'
        },
        'schedule': {
            'patternType': 'MINUTES',
            'pattern': '30',
            'startTime': '00:30:00'
        }
    }]

    resp = discovery.create(jdata)
    print(resp)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import logging
import argparse

import opsramp.binding


def connect():
    url = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    parser.add_argument(
        'uuid',
        type=str
    )
    ns = parser.parse_args()
    return ns


def create_command_script(targetcat):
    # Create a new RBA script in this category, with one parameter.
    # OpsRamp will prompt for a value for this parameter each time
    # you run this script in the UI and will pass the value in as $1
    # which you can see being used in the "payload" below.
    p1 = targetcat.mkParameter(
        name='venue',
        description='Where am I today?',
        datatype='STRING'
    )
    s1 = targetcat.mkScript(
        name='Hello <venue>',
        description='Stereotypical rock star intro',
        platforms=['LINUX'],
        execution_type='COMMAND',
        payload='echo "hello $1"',
        parameters=[p1]
    )
    print('creating new COMMAND script...')
    print(s1)
    resp = targetcat.create(s1)
    return resp


def create_python_script(targetcat):
    scriptfile = '/tmp/whatever-%s.py' % os.getpid()
    with open(scriptfile, 'w') as f:
        f.write('''#! /usr/bin/env python
from __future__ import print_function
print('hello world')
''')
    s1 = targetcat.mkScript(
        name='Python hello',
        description='Hello from Python land',
        platforms=['LINUX'],
        execution_type='PYTHON',
        script_name=os.path.basename(scriptfile),
        payload_file=scriptfile
    )
    print('creating new PYTHON script...')
    print(s1)
    resp = targetcat.create(s1)
    os.remove(scriptfile)
    return resp


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
    category_id = int(ns.uuid)

    tenant_id = os.environ['OPSRAMP_TENANT_ID']

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    targetcat = tenant.rba().categories().category(category_id)

    # A "COMMAND" script is a basic oneliner that doesn't require a shell.
    resp = create_command_script(targetcat)
    print(resp)

    resp = create_python_script(targetcat)
    print(resp)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A command line interface to OpsRamp that illustrates how to use
# this language binding as well as being useful in its own right.
#
# (c) Copyright 2019 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import json
import argparse

import opsramp.binding


# TODO make these optional command line parameters
OPSRAMP_URL = os.environ['OPSRAMP_URL']
OPSRAMP_KEY = os.environ['OPSRAMP_KEY']
OPSRAMP_SECRET = os.environ['OPSRAMP_SECRET']
OPSRAMP_TENANT_ID = os.environ['OPSRAMP_TENANT_ID']


def do_auth():
    return opsramp.binding.connect(OPSRAMP_URL, OPSRAMP_KEY, OPSRAMP_SECRET)


# Ideas for the cli syntax:
#
# ormpcli \
#   --tenant=client_2703 \
#   --endpoint=https://hpe-dev.api.try.opsramp.com \
#   --key='redacted' \
#   --secret='redacted' \
#   subtree area action
#
# ormpcli tenant agent script
# ormpcli tenant rba categories
# ormpcli tenant monitoring templates

def parse_args():
    parser = argparse.ArgumentParser(description='HPE cli for OpsRamp')
    parser.add_argument('subtree')
    parser.add_argument('area')
    parser.add_argument('action')
    return parser.parse_args()


def do_tenant_rba_action(rba, action):
    if action == 'categories':
        clist = rba.categories().get()
        print(json.dumps(clist))
    else:
        raise ValueError(rba, action)


def do_tenant_monitoring_action(monitoring, action):
    if action == 'templates':
        tplates = monitoring.templates()
        resp = tplates.search()
        print(resp['totalResults'], 'monitoring templates found')
    else:
        raise ValueError(monitoring, action)


def do_tenant_action(tenant, area, action):
    if area == 'rba':
        subapi = tenant.rba()
        return do_tenant_rba_action(subapi, action)
    elif area == 'monitoring':
        subapi = tenant.monitoring()
        return do_tenant_monitoring_action(subapi, action)
    elif area == 'agent' and action == 'script':
        contents = tenant.get_agent_script()
        print(contents)
    else:
        raise ValueError(tenant, area, action)


def do_action(ormp, subtree, area, action):
    if subtree == 'tenant':
        tenant = ormp.tenant(OPSRAMP_TENANT_ID)
        return do_tenant_action(tenant, area, action)
    else:
        raise ValueError(subtree, area, action)


def main():
    args = parse_args()
    ormp = do_auth()
    do_action(ormp, args.subtree, args.area, args.action)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
import requests_mock

import opsramp.binding


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.service_maps = self.client.service_maps()
        assert 'serviceGroups' in str(self.service_maps)

    def test_service_maps_get_root(self):
        group = self.service_maps
        thisid = 111111
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('search')
            m.get(url, json=expected, complete_qs=True)
            actual = group.get()
            assert actual == expected

    def test_service_maps_get_child(self):
        group = self.service_maps
        thisid = 222222
        expected = {'id': thisid}
        sgid = 'some-sgid'
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/childs/search' % sgid)
            m.get(url, json=expected, complete_qs=True)
            # test uuid parameter specifically.
            actual = group.get(uuid=sgid)
            assert actual == expected
            # test that uuid is the first parameter
            actual = group.get(sgid)
            assert actual == expected

    def test_service_maps_get_minimal(self):
        group = self.service_maps
        thisid = 333333
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('minimal')
            m.get(url, json=expected, complete_qs=True)
            actual = group.get(minimal=True)
            assert actual == expected

    def test_service_maps_create(self):
        group = self.service_maps
        thisid = 444444
        expected = {'id': thisid}
        fake_defn = {'name': 'fr dick byrne'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(definition=fake_defn)
            assert actual == expected

    def test_service_maps_update(self):
        group = self.service_maps
        thisid = 123456
        expected = {'id': thisid}
        fake_defn = {'name': 'fr todd unctious'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_service_maps_delete(self):
        group = self.service_maps
        thisid = 789012
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.delete(url, json=expected, complete_qs=True)
            actual = group.delete(uuid=thisid)
            assert actual == expected



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import yaml
import logging
import argparse

import opsramp.binding


def connect():
    url = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ['OPSRAMP_TENANT_ID']

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    group = tenant.integrations().instances()

    # Create new Email Alerts Integration...
    jdata = {
        'displayName': 'Test Email Alerts integration',
        'emailProps': [{
            'name': 'logz.io email alert',
            'identifier': '^\\[Alert\\]',
            'identifierSource': 'EMAIL_SUBJECT',
            'properties': [{
                'name': 'Alert State',
                'defaultValue': 'OK',
                'condition': {
                    'contentSource': 'EMAIL_SUBJECT',
                    'operator': 'BETWEEN',
                    'startValue': '\\(',
                    'endValue': ' severity\\)'
                },
                'propertyMappings': {
                    'attrValues': [{
                        'attrValue': 'Critical',
                        'thirdPartyAttrValue': 'Severe'
                    },
                        {
                        'attrValue': 'Warning',
                        'thirdPartyAttrValue': 'High'
                    },
                        {
                        'attrValue': 'Observed',
                        'thirdPartyAttrValue': 'Medium'
                    },
                        {
                        'attrValue': 'Info',
                        'thirdPartyAttrValue': 'Low'
                    },
                        {
                        'attrValue': 'Ok',
                        'thirdPartyAttrValue': 'Info'
                    }]
                }
            },
                {
                'name': 'Service Name',
                'defaultValue': 'whatever',
                'condition': {
                    'contentSource': 'DEFAULT_VALUE'
                }
            },
                {
                'name': 'Device Host Name',
                'defaultValue': 'whatever',
                'condition': {
                    'contentSource': 'DEFAULT_VALUE'
                }
            }]
        }]
    }

    print('create integration using this payload')
    print(yaml.dump(jdata, default_flow_style=False))

    resp = group.create('Email Alerts', jdata)
    print('result:')
    print(yaml.dump(resp, default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import base64
import copy
import unittest
import requests_mock

import opsramp.binding
from opsramp.integrations import Instances


class InstancesTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.integs = self.client.integrations()
        assert 'Integrations' in str(self.integs)

    def test_itypes(self):
        group = self.integs.itypes()
        thisid = 111111
        expected = {'id': thisid}
        pattern = 'whatever'
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('search?%s' % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
            assert actual == expected
        # check the compatibility function would return the
        # same type of object, so there's no need to repeat
        # all of the tests for it.
        assert isinstance(self.integs.available(), type(group))

    def test_instances(self):
        group = self.integs.instances()
        thisid = 222222
        expected = {'id': thisid}
        pattern = 'whatever'
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('search?%s' % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
            assert actual == expected
        # check the compatibility function would return the
        # same type of object, so there's no need to repeat
        # all of the tests for it.
        assert isinstance(self.integs.installed(), type(group))

    def test_instance_kubernetes_configuration(self):
        group = self.integs.instances()
        thisid = 123456
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/configFile/Kubernetes' % thisid)
            m.get(url, json=expected, complete_qs=True)
            actual = group.get_kubernetes_configuration(uuid=thisid)
            assert actual == expected

    def test_instance_create(self):
        group = self.integs.instances()
        thisid = 789012
        expected = {'id': thisid}
        name = 'unit-test-integration'
        fake_defn = {'name': 'fr larry duff'}
        with requests_mock.Mocker() as m:
            url = group.creator_api.compute_url(name)
            m.post(url, json=expected, complete_qs=True)
            actual = group.create(type_name=name, definition=fake_defn)
            assert actual == expected

    def test_instance_update(self):
        group = self.integs.instances()
        thisid = 123456
        expected = {'id': thisid}
        fake_defn = {'name': 'fr paul stone'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.update(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_instance_enable(self):
        group = self.integs.instances()
        thisid = 789012
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/enable' % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.enable(uuid=thisid)
            assert actual == expected

    def test_instance_disable(self):
        group = self.integs.instances()
        thisid = 345678
        expected = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/disable' % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.disable(uuid=thisid)
            assert actual == expected

    def test_instance_delete(self):
        group = self.integs.instances()
        thisid = 999999
        url = group.api.compute_url('%s' % thisid)

        expected_response = {'id': thisid}

        # Test that the delete function inserts a default reason
        # if we don't provide one.
        expected_send = {"uninstallReason": "<Not specified>"}
        with requests_mock.Mocker() as m:
            adapter = m.delete(url, json=expected_response, complete_qs=True)
            actual_response = group.delete(uuid=thisid)
            assert adapter.last_request.json() == expected_send
            assert actual_response == expected_response

        # Test that the delete function sends the reason we specified.
        delete_reason = 'Totally fake reason to delete something'
        expected_send = {"uninstallReason": delete_reason}
        with requests_mock.Mocker() as m:
            adapter = m.delete(url, json=expected_response, complete_qs=True)
            actual_response = group.delete(
                uuid=thisid,
                uninstall_reason=delete_reason
            )
            assert adapter.last_request.json() == expected_send
            assert actual_response == expected_response

    def test_instance_notifier(self):
        group = self.integs.instances()
        thisid = 901234
        expected = {'id': thisid}
        fake_defn = {'name': 'fr fintan stack'}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('%s/notifier' % thisid)
            m.post(url, json=expected, complete_qs=True)
            actual = group.notifier(uuid=thisid, definition=fake_defn)
            assert actual == expected

    def test_instance_auth_type(self):
        group = self.integs.instances()
        thisid = 567890
        url = group.api.compute_url('%s/inbound/authentication' % thisid)
        for key in 'OAUTH2', 'WEBHOOK', 'BASIC':
            expected = {'type': key}
            with requests_mock.Mocker() as m:
                m.post(url, json=expected, complete_qs=True)
                actual = group.set_auth_type(uuid=thisid, auth_type=key)
                assert actual == expected
        with requests_mock.Mocker() as m:
            with self.assertRaises(AssertionError):
                group.set_auth_type(uuid=thisid, auth_type='unit test value')
            assert m.call_count == 0

    def base_display_name(self, fn):
        with self.assertRaises(AssertionError):
            fn(display_name=None)
        testname = 'Unit test 1'
        actual = fn(display_name=testname)
        expected = {
            'displayName': testname
        }
        assert actual == expected

    def base_logo(self, fn):
        testname = 'Unit test 2'
        testfile = 'README.md'
        with self.assertRaises(IOError):
            fn(
                display_name=testname,
                logo_fname=testfile + '.that.doesnt.exist'
            )
        actual = fn(
            display_name=testname,
            logo_fname=testfile
        )
        encoded = actual['logo']['file']
        cleartext = base64.b64decode(encoded)
        with open(testfile, 'rb') as f:
            content = f.read()
        assert content == cleartext

        expected = {
            'displayName': testname,
            'logo': {
                'name': testfile,
                'file': encoded
            }
        }
        assert actual == expected

    def test_base(self):
        self.base_display_name(Instances.mkBase)
        self.base_logo(Instances.mkBase)

    def test_emailalert(self):
        self.base_display_name(Instances.mkEmailAlert)
        self.base_logo(Instances.mkEmailAlert)

    def test_custom(self):
        self.base_display_name(Instances.mkCustom)
        self.base_logo(Instances.mkCustom)
        with self.assertRaises(AssertionError):
            Instances.mkCustom(
                display_name='bad auth type',
                inbound_auth_type='BAD UNIT TEST VALUE'
            )
        # Now a variant that should work.
        testname = 'Custom integration unit test'
        fakeuuid = 'deadbeef'
        auth_type = 'OAUTH2'
        actual = Instances.mkCustom(
            display_name=testname,
            parent_uuid=fakeuuid,
            inbound_auth_type=auth_type
        )
        expected = {
            'displayName': testname,
            'parentIntg': {
                'id': fakeuuid
            },
            'inboundConfig': {
                'authentication': {'authType': auth_type}
            }
        }
        assert actual == expected
        # exercise the auth_type helper while we have a suitable struct.
        atypes = Instances.auth_type(actual)
        assert atypes == (auth_type, None)

    def test_ARM(self):
        testname = 'Azure ARM integration unit test'
        subscription_id = '123456789'
        tenant_id = 'Some random Azure tenant'
        client_id = 'Azure client shtuff'
        secret_key = 'Open Sesame I Say!'
        actual = Instances.mkAzureARM(
            display_name=testname,
            arm_subscription_id=subscription_id,
            arm_tenant_id=tenant_id,
            arm_client_id=client_id,
            arm_secret_key=secret_key
        )
        expected = {
            'displayName': testname,
            'credential': {
                'credentialType': 'AZURE',
                'AzureType': 'ARM',
                'SubscriptionId': subscription_id,
                'TenantId': tenant_id,
                'ClientID': client_id,
                'SecretKey': secret_key
            }
        }
        assert actual == expected

    def test_ASM(self):
        testname = 'Azure ASM integration unit test'
        subscription_id = 'ABCDEFGHIJKLMNOP'
        mgmt_cert = 'Azure management cert'
        keystore_pass = 'Abrakedabra'
        actual = Instances.mkAzureASM(
            display_name=testname,
            arm_subscription_id=subscription_id,
            arm_mgmt_cert=mgmt_cert,
            arm_keystore_pass=keystore_pass
        )
        expected = {
            'displayName': testname,
            'credential': {
                'credentialType': 'AZURE',
                'AzureType': 'ASM',
                'SubscriptionId': subscription_id,
                'ManagementCertificate': mgmt_cert,
                'KeystorePassword': keystore_pass
            }
        }
        assert actual == expected

    def test_valuemap(self):
        expected = [
            {
                'attrValue': 'client_113',
                'tenantAttrValue': '22cdbc5bb401d737b088c9'
            },
            {
                'attrValue': 'client_843',
                'tenantAttrValue': '66cdbc5bb401d737b088c9'
            }
        ]
        actual = Instances.mkValueMap(
            ('attrValue', 'tenantAttrValue'),
            (('client_113', '22cdbc5bb401d737b088c9'),
             ('client_843', '66cdbc5bb401d737b088c9'))
        )
        assert actual == expected

        expected = [
            {
                'attrValue': 'Fido',
                'thirdPartyAttrValue': 'His Royal Fidoness'
            },
            {
                'attrValue': 'brown',
                'thirdPartyAttrValue': 'Autumn Sunset'
            }
        ]
        actual = Instances.mkValueMap(
            ('attrValue', 'thirdPartyAttrValue'),
            (('Fido', 'His Royal Fidoness'),
             ('brown', 'Autumn Sunset'))
        )
        assert actual == expected

    def test_basenotifier(self):
        expected = {
            'type': 'REST_API',
            'baseURI': 'www.example.com',
            'authType': 'OAUTH2',
            'grantType': 'CLIENT_CREDENTIALS',
            'accessTokenURI': 'www.example.com/creds',
            'apiKey': '6h67PAAFscVPMwhQZFcshpcqN5b6pyU9',
            'apiSecret': 'totalandcompletenonsense'
        }
        actual = Instances.mkBaseNotifier(
            expected['type'], expected['baseURI'],
            access_token_uri=expected['accessTokenURI'],
            api_key=expected['apiKey'],
            api_secret=expected['apiSecret']
        )
        assert actual == expected

        expected = {
            'type': 'SOAP_API',
            'baseURI': 'www.example.com',
            'authType': 'NONE'
        }
        actual = Instances.mkBaseNotifier(
            expected['type'], expected['baseURI'],
            auth_type=expected['authType']
        )
        assert actual == expected

        expected = {
            'type': 'REST_API',
            'baseURI': 'www.example.com/whatevs',
            'authType': 'BASIC',
            'grantType': 'PASSWORD',
            'userName': 'elvis',
            'password': 'graceland'
        }
        actual = Instances.mkBaseNotifier(
            expected['type'], expected['baseURI'],
            auth_type=expected['authType'],
            grant_type=expected['grantType'],
            user_name=expected['userName'],
            password=expected['password']
        )
        assert actual == expected

    def test_redaction(self):
        original = {
            'inboundConfig': {
                'authentication': {
                    'token': 'shakespeare'
                }
            },
            'outboundConfig': {
                'authentication': {
                    'apiKeyPairs': [{
                        'char1': 'juliet',
                        'char2': 'goneril'
                    }]
                }
            }
        }
        # make a copy and check that it really is a copy.
        tvalues = copy.deepcopy(original)
        assert tvalues is not original
        assert tvalues == original
        # the redact function should modify the dict in place, not copy it.
        result = Instances.redact_response(tvalues)
        assert result is tvalues
        # check the redaction did what we expected.
        original['inboundConfig']['authentication'][
            'token'] = 'REDACTED'
        original['outboundConfig']['authentication'][
            'apiKeyPairs'][0]['char1'] = 'REDACTED'
        original['outboundConfig']['authentication'][
            'apiKeyPairs'][0]['char2'] = 'REDACTED'
        assert result is not original
        assert result == original



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# binding.py
# Defines the primary entry points for callers of this library.
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

from opsramp.base import ApiObject
from opsramp.api import ORapi
from opsramp.globalconfig import GlobalConfig
from opsramp.tenant import Tenant
from opsramp.metrics import MetricsApi


def connect(url, key, secret):
    auth_url = url + '/auth/oauth/token'
    auth_hdrs = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Accept': 'application/json,application/xml'
    }
    body = 'grant_type=client_credentials&' \
           'client_id=%s&' \
           'client_secret=%s' % (key, secret)
    ao = ApiObject(auth_url, auth_hdrs)
    auth_resp = ao.post(data=body)
    token = auth_resp['access_token']
    return Opsramp(url, token)


class Opsramp(ORapi):
    def __init__(self, url, token):
        self.auth = {
            'Authorization': 'Bearer ' + token,
            'Accept': 'application/json,application/xml'
        }
        apiobject = ApiObject(url + '/api/v2', self.auth)
        super(Opsramp, self).__init__(apiobject)

    def __str__(self):
        return '%s %s' % (str(type(self)), self.api)

    def config(self):
        return GlobalConfig(self)

    def tenant(self, name):
        return Tenant(self, name)

    def metrics(self):
        return MetricsApi(self)



#!/usr/bin/env python
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
from requests import codes as http_status
from mock import MagicMock
import requests_mock

# Note we are deliberately using simplejson (instead of json) because
# that's what the requests module uses and we want to raise the same
# types of exceptions, not exceptions from the generic "json" module.
import simplejson

import opsramp.base


class FakeResp(object):
    def __init__(self, content, request):
        self.status_code = http_status.OK
        self.content = content
        self.text = simplejson.dumps(self.content)
        self.json_fail = False
        self.request = request

    def json(self):
        content = self.content
        if self.json_fail:
            # deliberately cause a parsing exception
            content = 'deliberately bad JSON for unit test purposes!!'
        return content


class ApiObjectTest(unittest.TestCase):
    def setUp(self):
        self.fake_url = 'mock://api.example.com'
        self.fake_token = 'ffffffffffffffff'
        self.fake_auth = {
            'Authorization': 'Bearer %s' % self.fake_token,
            'Accept': 'application/json'
        }
        self.ao = opsramp.base.ApiObject(
            self.fake_url,
            self.fake_auth.copy()
        )
        assert 'ApiObject' in str(self.ao)

    def test_shared_session(self):
        '''All clones should share the requests session'''
        ao2 = self.ao.clone()
        assert ao2.session is self.ao.session

    def test_compute_url(self):
        # some value, doesn't really matter.
        suffix = 'unit/test/value'
        expected = self.ao.baseurl + '/' + suffix
        assert self.ao.compute_url(suffix) == expected
        # empty string and None should both return the root path.
        rootdir = self.ao.baseurl
        assert self.ao.compute_url() == rootdir
        assert self.ao.compute_url('/') == rootdir
        assert self.ao.compute_url('') == rootdir
        assert self.ao.compute_url(None) == rootdir

    # We're not testing an exhaustive set of suffix patterns here because
    # that is already being done by the PathTracker unit tests.
    def test_cd(self):
        # some value, doesn't really matter.
        suffix = 'unit/test/cd'
        expected = self.ao.compute_url() + '/' + suffix
        actual = self.ao.cd(suffix)
        assert actual == expected
        assert self.ao.compute_url() == expected
        # check the root path works.
        rootdir = self.ao.baseurl
        self.ao.cd('/random/place1')
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd('/') == rootdir
        assert self.ao.compute_url() == rootdir
        # no args should default to the root path.
        self.ao.cd('/random/place0')
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd() == rootdir
        assert self.ao.compute_url() == rootdir
        # empty string should default to the root path.
        self.ao.cd('/random/place2')
        assert self.ao.compute_url() != rootdir
        assert self.ao.cd('') == rootdir
        assert self.ao.compute_url() == rootdir
        # None should default to the root path.
        self.ao.cd('/random/place3')
        assert self.ao.cd(None) == rootdir
        assert self.ao.compute_url() == rootdir

    # We're not testing an exhaustive set of suffix patterns here because
    # that is already being done by the PathTracker unit tests.
    def test_pushpopd(self):
        suffix = 'unit/test/pushd'
        base = self.ao.compute_url()
        expected = base + '/' + suffix
        actual = self.ao.pushd(suffix)
        assert actual == expected
        assert self.ao.compute_url() == expected
        actual = self.ao.popd()
        assert actual == base
        assert self.ao.compute_url() == base

    def test_headers(self):
        expected = self.fake_auth.copy()
        actual = self.ao.prep_headers({})
        assert actual == expected

        expected = self.fake_auth.copy()
        custom = {
            'color': 'pink',
            'distance': 'far'
        }
        expected.update(custom)
        actual = self.ao.prep_headers(custom)
        assert actual == expected

        # The caller is supposed to be able to override any of the standard
        # auth headers by providing its own value. Check this works.
        expected = self.fake_auth.copy()
        key = 'Authorization'
        assert key in expected
        testvalue = 'open sesame'
        assert expected[key] != testvalue
        custom[key] = testvalue
        actual = self.ao.prep_headers(custom)
        assert actual[key] == testvalue
        expected.update(custom)
        assert actual == expected

    def test_results(self):
        # Test successful response

        with requests_mock.mock() as m:
            expected = {'hello': 'world'}
            m.get(self.fake_url, json=expected, status_code=http_status.OK)

            faked_response = FakeResp(
                content=expected,
                request=self.ao.session.get(self.fake_url).request
            )
            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is dict
            assert actual == expected

        # Test response where result is not valid JSON
        with requests_mock.mock() as m:
            expected = 'deliberately bad JSON for unit test purposes!!'
            m.get(self.fake_url, text=expected)

            faked_response = FakeResp(
                content=['nothing'],
                request=self.ao.session.get(self.fake_url).request
            )

            faked_response.json_fail = True

            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is str
            assert actual == expected

        # Test that exception is thrown if unexpected HTTP status code returned
        with requests_mock.mock() as m:
            m.get(
                self.fake_url,
                status_code=http_status.BAD_REQUEST,
                text='Pretending to fail horribly'
            )

            faked_response = FakeResp(
                content=['nothing'],
                request=self.ao.session.get(self.fake_url).request
            )
            faked_response.status_code = http_status.BAD_REQUEST

            failed = False
            try:
                actual = self.ao.process_result(self.fake_url, faked_response)
            except RuntimeError as e:
                print(e)
                failed = True
            assert failed

    def test_paginated_results(self):
        # Define three pages of test data that return the following result
        # data:
        # Page 1: 1-10
        # Page 2: 11-20
        # Page 3: 21-30
        # If the pagination works, the result should be the range 1-30.
        request_responses = []
        for result_page in range(3):
            result_data = list(
                range((10 * result_page) + 1, (10 * result_page) + 11)
            )

            response_body = {
                'results': result_data,
                'totalResults': 30,
                'pageNo': result_page + 1,
                'pageSize': 10,
                'nextPage': (result_page < 2),
                'previousPageNo': result_page,
                'descendingOrder': False
            }

            response = {
                'status_code': http_status.OK,
                'json': response_body,
            }

            request_responses.append(response)

        # Build up the list of expected result data based on what we're feeding
        # in to the mock.
        expected = []
        for response in request_responses:
            expected = expected + response['json']['results']

        with requests_mock.mock() as m:
            # Feed the mock with the three responses it should return
            m.get(self.fake_url, request_responses)

            # Build the initial (faked) response based on the above
            faked_response = FakeResp(
                content=request_responses[0]['json'],
                request=self.ao.session.get(self.fake_url).request
            )
            actual = self.ao.process_result(self.fake_url, faked_response)
            assert type(actual) is dict

            # So that no existing code is broken, the behaviour emulates the
            # existing behaviour of the API except that it acts as if the data
            # was in one big page.
            assert actual['results'] == expected
            assert actual['totalResults'] == len(expected)
            assert actual['nextPage'] is False
            assert actual['pageNo'] == 1
            assert actual['previousPageNo'] == 0

        # Test that if the request is not a HTTP GET, complain:
        fake_get_request = MagicMock()
        fake_get_request.method = 'FAKE'
        expected = ['request is not GET']
        actual = self.ao.collate_pages(fake_get_request, expected)
        assert actual == expected

        # Test that it handles the lack of a "results" fields in the data
        # correctly
        fake_get_request.method = 'GET'
        expected = ['request does not contain results key']
        actual = self.ao.collate_pages(fake_get_request, expected)
        assert actual == expected

        # Test that if the next page is not retrieved successfully, an empty
        # result is returned.
        with requests_mock.mock() as m:
            m.get(
                self.fake_url,
                status_code=http_status.BAD_REQUEST,
                json=['should not be seen']
            )

            fake_get_request = MagicMock()
            fake_get_request.method = 'GET'
            fake_get_request.url = self.fake_url

            page_1_data = {
                'results': ['should not be seen'],
                'nextPage': True,
                'pageNo': 1
            }

            actual = self.ao.collate_pages(fake_get_request, page_1_data)
            assert actual['results'] == []
            assert actual['totalResults'] == 0
            assert actual['pageSize'] == 0

    def test_get(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = 'unit test get result'
            m.get(url, text=expected)
            actual = self.ao.get()
            assert actual == expected

    def test_put(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = 'unit test put result'
            m.put(url, text=expected)
            actual = self.ao.put()
            assert actual == expected

    def test_post(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = 'unit test post result'
            m.post(url, text=expected)
            actual = self.ao.post()
            assert actual == expected

    def test_delete(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = 'unit test delete result'
            m.delete(url, text=expected)
            actual = self.ao.delete()
            assert actual == expected

    def test_patch(self):
        with requests_mock.Mocker() as m:
            url = self.ao.compute_url()
            expected = 'unit test patch result'
            m.patch(url, text=expected)
            actual = self.ao.patch()
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
import requests_mock

import opsramp.binding


class MonitoringTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.monitoring = self.client.monitoring()
        assert 'Monitoring' in str(self.monitoring)

    def test_templates(self):
        group = self.monitoring.templates()
        thisid = 888888
        expected = {'id': thisid}
        pattern = 'whatever'
        with requests_mock.Mocker() as m:
            url = group.api.compute_url('search?%s' % pattern)
            m.get(url, json=expected, complete_qs=True)
            actual = group.search(pattern)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
import requests_mock

import opsramp.binding
import opsramp.resources


class StaticsTest(unittest.TestCase):
    def test_list2ormp_passthru(self):
        good1 = {'results': 'exist', 'totalResults': 12}
        result = opsramp.resources.list2ormp(good1)
        assert result is good1

    def test_list2ormp_conversion(self):
        list_form = [1, 2, 3, 4, 5, 6, 7, 8]
        count = len(list_form)
        canonical_form = {
            'totalResults': count,
            'pageSize': count,
            'totalPages': 1,
            'pageNo': 1,
            'previousPageNo': 0,
            'nextPage': False,
            'descendingOrder': False,
            'results': list_form
        }
        result = opsramp.resources.list2ormp(list_form)
        assert result == canonical_form

    def test_list2ormp_invalid(self):
        bad_ones = (
            3.14159,
            'string not allowed',
            {},
            {'big cow': 'close', 'small cow': 'far away'},
            {'results': 'not enough on their own'},
            {'totalResults': 0, 'noresults': 'bad'}
        )
        for bad1 in bad_ones:
            with self.assertRaises(AssertionError):
                opsramp.resources.list2ormp(bad1)


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

    def test_create(self):
        group = self.client.resources()
        fake_create_json = {
            'resourceName': 'ABBY-PC',
            'hostName': 'ABBY-PC',
            'aliasName': 'Server PC',
            'resourceType': 'server',
            'os': 'Ubuntu 14.04.6 LTS',
            'serialNumber': '1234-5678-901234'
        }
        thisid = 333333
        fake_result = {'id': thisid}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url()
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.create(definition=fake_create_json)
            assert actual == fake_result

    def test_update(self):
        group = self.client.resources()
        fake_resource_id = '123456'
        fake_update_json = {
            'os': 'Ubuntu 18.04.4 LTS'
        }
        fake_result = {'id': fake_resource_id}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(fake_resource_id)
            m.post(url, json=fake_result, complete_qs=True)
            actual = group.update(
                uuid=fake_resource_id,
                definition=fake_update_json
            )
            assert actual == fake_result

    def test_delete(self):
        group = self.client.resources()
        fake_resource_id = '789012'
        fake_result = {'id': fake_resource_id}
        with requests_mock.Mocker() as m:
            url = group.api.compute_url(fake_resource_id)
            m.delete(url, json=fake_result, complete_qs=True)
            actual = group.delete(uuid=fake_resource_id)
            assert actual == fake_result

    def test_search(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_search_pattern = 'queryString=agentInstalled:true'
        fake_raw_result = ['unit', 'test', 'search', 'values']
        count = len(fake_raw_result)
        fake_cooked_result = {
            'totalResults': count,
            'pageSize': count,
            'totalPages': 1,
            'pageNo': 1,
            'previousPageNo': 0,
            'nextPage': False,
            'descendingOrder': False,
            'results': fake_raw_result
        }
        with requests_mock.Mocker() as m:
            url_suffix = 'search?{0}'.format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.search(pattern=fake_search_pattern)
            assert actual == fake_cooked_result

    def test_minimal(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_search_pattern = 'queryString=agentInstalled:true'
        fake_raw_result = ['unit', 'test', 'minimal', 'result']
        count = len(fake_raw_result)
        fake_cooked_result = {
            'totalResults': count,
            'pageSize': count,
            'totalPages': 1,
            'pageNo': 1,
            'previousPageNo': 0,
            'nextPage': False,
            'descendingOrder': False,
            'results': fake_raw_result
        }
        with requests_mock.Mocker() as m:
            url_suffix = 'minimal?{0}'.format(fake_search_pattern)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.minimal(pattern=fake_search_pattern)
            assert actual == fake_cooked_result

    def test_applications(self):
        # The OpsRamp "resources" API returns flat lists of results
        # instead of the usual multi-value result struct, for no
        # apparent reason. Our Python classes are supposed to handle
        # that and return a normal result struct so that the caller
        # doesn't need to know. Check that it's doing this.
        group = self.client.resources()
        fake_resource_id = '789012'
        fake_raw_result = ['unit', 'test', 'applications', 'result']
        count = len(fake_raw_result)
        fake_cooked_result = {
            'totalResults': count,
            'pageSize': count,
            'totalPages': 1,
            'pageNo': 1,
            'previousPageNo': 0,
            'nextPage': False,
            'descendingOrder': False,
            'results': fake_raw_result
        }
        with requests_mock.Mocker() as m:
            url_suffix = '{0}/applications'.format(fake_resource_id)
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_raw_result, complete_qs=True)
            actual = group.applications(uuid=fake_resource_id)
            assert actual == fake_cooked_result

    def test_availability(self):
        group = self.client.resources()
        # this a unix epoch timestamp. its actual value is not important, so
        # i just looked at the clock when i first wrote this function.
        fake_start = 1584727225
        fake_end = fake_start + (60 * 24)
        fake_resource_id = '789012'
        fake_result = ['unit', 'test', 'availability', 'result']
        with requests_mock.Mocker() as m:
            url_suffix = '{0}/availability?startTime={1}&endTime={2}'.format(
                fake_resource_id, fake_start, fake_end
            )
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_result, complete_qs=True)
            actual = group.availability(
                uuid=fake_resource_id,
                start_epoch=fake_start,
                end_epoch=fake_end
            )
            assert actual == fake_result

    def test_get_templates(self):
        group = self.client.resources()
        fake_resource_id = '789012'
        fake_result = ['unit', 'test', 'availability', 'result']
        with requests_mock.Mocker() as m:
            url_suffix = '{0}/templates/search?{1}'.format(
                fake_resource_id, "fake_pattern"
            )
            url = group.api.compute_url(url_suffix)
            m.get(url, json=fake_result, complete_qs=True)
            actual = group.get_templates(
                uuid=fake_resource_id,
                pattern="fake_pattern"
            )
            assert actual['results'] == fake_result



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import logging
import argparse

import opsramp.binding


def connect():
    url = os.environ['OPSRAMP_URL']
    key = os.environ['OPSRAMP_KEY']
    secret = os.environ['OPSRAMP_SECRET']
    return opsramp.binding.connect(url, key, secret)


def parse_argv():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-d', '--debug',
        action='store_true'
    )
    ns = parser.parse_args()
    return ns


def main():
    ns = parse_argv()
    if ns.debug:
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)

    tenant_id = os.environ['OPSRAMP_TENANT_ID']

    ormp = connect()
    tenant = ormp.tenant(tenant_id)
    tid_list = [(tenant_id, '')]
    if not tenant.is_client():
        children = tenant.clients().get()
        cdata = [(x['uniqueId'], x['name']) for x in children]
        tid_list.extend(cdata)

    for uuid, name in tid_list:
        child = ormp.tenant(uuid)
        categs = child.rba().categories()
        clist = categs.get()
        print('{0} "{1}"'.format(uuid, name))
        for cdata in clist:
            show_contents(categs, cdata, 1)


def show_contents(categs, cdata, indent):
    print('{0} category {1:05d} {2}'.format(
        '..' * indent,
        cdata['id'],
        cdata['name']
    ))
    # get the list of scripts in the category.
    thiscat = categs.category(cdata['id'])
    try:
        scripts = thiscat.get()
    except RuntimeError as e:
        # we have encountered broken trees in the field so guard against it.
        print(e)
        scripts = []
    # print out a short description of each script.
    for s in scripts:
        print('{0}. script {1:06d} {2} {3} "{4}" "{5}"'.format(
            '..' * indent,
            s['id'],
            s['platforms'],
            s['executionType'],
            s['name'],
            s['description']
        ))
    # recurse into any embedded categories.
    if 'childs' in cdata:
        for child in cdata['childs']:
            show_contents(categs, child, indent + 1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# sites.py
# Classes dealing directly with OpsRamp Sites.
# Sites are used to organise resources by geographical location.
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
from opsramp.api import ORapi


class Sites(ORapi):
    def __init__(self, parent):
        super(Sites, self).__init__(parent.api, 'sites')

    def get(self, suffix='/minimal'):
        return self.api.get(suffix)

    def create(self, definition):
        return self.api.post('', json=definition)

    def update(self, uuid, definition):
        return self.api.post('%s' % uuid, json=definition)

    def delete(self, uuid):
        return self.api.delete('%s' % uuid)



#!/usr/bin/env python
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import unittest
import requests_mock

from opsramp.api import ORapi
from opsramp.rba import Category
import opsramp.binding


class StaticsTest(unittest.TestCase):
    def setUp(self):
        # Several of the tests need a filename, its contents and
        # the base64 encoded version, so compute all of that up
        # front and store the data for later use. The actual
        # contents are not important so use any file that we know
        # will exist.
        self.testfile = 'setup.py'
        with open(self.testfile, 'rb') as f:
            self.content_raw = f.read()
        # check it's not empty.
        assert self.content_raw
        self.content_64 = ORapi.b64encode_payload(self.testfile)

    def test_mkAttachment(self):
        tvalues = {
            'name': self.testfile,
            'content': self.content_raw
        }
        actual = Category.mkAttachment(
            name=tvalues['name'],
            payload=tvalues['content']
        )
        expected = {
            'name': tvalues['name'],
            'file': tvalues['content']
        }
        assert actual == expected

    def test_mkBadParameter(self):
        with self.assertRaises(AssertionError):
            Category.mkParameter(
                name='pname', description='pdesc', datatype='STRING',
                optional=True
                # deliberately missing default value
            )

    def test_mkGoodParameter(self):
        tvalues = {
            'name': 'venue',
            'description': 'Where am I today?',
            'type': 'STRING'
        }
        actual = Category.mkParameter(
            name=tvalues['name'],
            description=tvalues['description'],
            datatype=tvalues['type']
        )
        expected = {
            'name': tvalues['name'],
            'description': tvalues['description'],
            'dataType': tvalues['type'],
            'type': 'REQUIRED',
            'defaultValue': None
        }
        assert actual == expected

    def test_mkBadScript(self):
        tvalues = {
            'name': 'sname',
            'description': 'pdesc',
            'platforms': ['LINUX']
        }
        with self.assertRaises(AssertionError):
            Category.mkScript(
                execution_type='Firing squad is not valid',
                name=tvalues['name'],
                description=tvalues['description'],
                platforms=tvalues['platforms'],
                payload_file=self.testfile
            )
        with self.assertRaises(AssertionError):
            Category.mkScript(
                execution_type='COMMAND',
                name=tvalues['name'],
                description=tvalues['description'],
                platforms=tvalues['platforms'],
                payload='if there is a payload',
                payload_file='cannot have payload_file as well'
            )

    def test_mkGoodCommandScript(self):
        p1 = Category.mkParameter(
            name='venue',
            description='Where am I today?',
            datatype='STRING'
        )
        tvalues = {
            'name': 'Hello <venue>',
            'description': 'Stereotypical rock star intro',
            'platforms': ['LINUX'],
            'type': 'COMMAND',
            'payload': 'echo "hello $1"',
            'parameters': [p1]
        }
        actual = Category.mkScript(
            name=tvalues['name'],
            description=tvalues['description'],
            platforms=tvalues['platforms'],
            execution_type=tvalues['type'],
            payload=tvalues['payload'],
            parameters=tvalues['parameters']
        )
        expected = {
            'name': tvalues['name'],
            'description': tvalues['description'],
            'platforms': tvalues['platforms'],
            'executionType': tvalues['type'],
            'command': tvalues['payload'],
            'parameters': tvalues['parameters']
        }
        assert actual == expected

    def test_mkGoodPythonScript(self):
        tvalues = {
            'name': 'Hello <venue>',
            'description': 'Python rock star intro',
            'platforms': ['LINUX'],
            'type': 'PYTHON',
            'process_name': 'jabberwocky',
            'service_name': 'silmarilion',
            'install_timeout': 'a really long time'
        }
        actual = Category.mkScript(
            name=tvalues['name'],
            description=tvalues['description'],
            platforms=tvalues['platforms'],
            execution_type=tvalues['type'],
            script_name=self.testfile,
            payload_file=self.testfile,
            install_timeout=tvalues['install_timeout'],
            process_name=tvalues['process_name'],
            service_name=tvalues['service_name']
        )
        expected = {
            'name': tvalues['name'],
            'description': tvalues['description'],
            'platforms': tvalues['platforms'],
            'executionType': tvalues['type'],
            'attachment': {
                'name': self.testfile,
                'file': self.content_64
            },
            'installTimeout': tvalues['install_timeout'],
            'processName': tvalues['process_name'],
            'serviceName': tvalues['service_name']
        }
        assert actual == expected


class ApiTest(unittest.TestCase):
    def setUp(self):
        fake_url = 'mock://api.example.com'
        fake_token = 'unit-test-fake-token'
        self.ormp = opsramp.binding.Opsramp(fake_url, fake_token)

        self.fake_client_id = 'client_for_unit_test'
        self.client = self.ormp.tenant(self.fake_client_id)
        assert self.client.is_client()

        self.rba = self.client.rba()
        assert 'Rba' in str(self.rba)

    def test_categories(self):
        group = self.rba.categories()

        parent = {'id': 123456, 'name': 'unit-test-parent-category'}
        child = {'id': 789012, 'name': 'unit-test-child-category'}

        with requests_mock.Mocker() as m:
            # we will be doing two posts so mock both results now.
            url = group.api.compute_url()
            m.post(url, [{'json': parent}, {'json': child}], complete_qs=True)

            # Create a category with no parent ID specified
            actual = group.create(
                name=parent['name']
            )
            assert actual == parent

            # Create a category with parent ID specified
            actual = group.create(
                name=child['name'],
                parent_uuid=parent['id']
            )
            assert actual == child

    def test_delete_category(self):
        category_group = self.rba.categories()
        assert category_group
        categoryID = 12345
        expected = ''
        with requests_mock.Mocker() as m:
            url = category_group.api.compute_url(categoryID)
            m.delete(url, text=expected, complete_qs=True)
            actual = category_group.delete(uuid=categoryID)
            assert actual == expected

    def test_update_category(self):
        category_group = self.rba.categories()
        assert category_group
        categoryID = 12345
        updated_name = 'some new name'
        expected = {
            'id': categoryID,
            'name': updated_name
        }

        with requests_mock.Mocker() as m:
            url = category_group.api.compute_url()
            m.put(url, json=expected, complete_qs=True)
            actual = category_group.update(categoryID, {
                'name': updated_name
            })
            assert actual == expected

    def test_create_script(self):
        this1 = self.rba.categories().category('unit-test-1')
        assert this1
        expected = {'unit': 'test'}
        assert expected
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url()
            m.post(url, json=expected, complete_qs=True)
            actual = this1.create(
                definition=expected
            )
            assert actual == expected

    def test_update_script(self):
        this1 = self.rba.categories().category('unit-test-1')
        assert this1
        scriptId = 12345
        expected = {'unit': 'test'}
        assert expected
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url(scriptId)
            m.post(url, json=expected, complete_qs=True)
            actual = this1.update(
                uuid=scriptId, definition=expected
            )
            assert actual == expected

    def test_delete_script(self):
        this1 = self.rba.categories().category('unit-test-1')
        assert this1
        scriptId = 67890
        expected = ''
        with requests_mock.Mocker() as m:
            url = this1.api.compute_url(scriptId)
            m.delete(url, text=expected, complete_qs=True)
            actual = this1.delete(uuid=scriptId)
            assert actual == expected



#!/usr/bin/env python
#
# (c) Copyright 2019-2021 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()
    long_description_content_type = 'text/markdown'

with open('requirements.txt', 'r') as fh:
    requirements = []
    for line in fh.readlines():
        line = line.strip()
        if line[0] != '#':
            requirements.append(line)

setuptools.setup(
    name='python-opsramp',
    python_requires='>=2.7',
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    author='HPE GreenLake CSO',
    author_email='eemz@hpe.com',
    description='Python language binding for the Opsramp API',
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    keywords='opsramp',
    url='https://github.com/HewlettPackard/python-opsramp',
    license='Apache 2.0',
    packages=setuptools.find_packages(exclude=['contrib', 'docs', 'tests']),
    classifiers=[
        'Development Status :: 4 - Beta ',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Software Development :: Libraries',
        'Operating System :: OS Independent',
    ],
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'ormpcli=opsramp.cli:main',
        ],
    }
)



#!/usr/bin/env python
#
# A minimal Python language binding for the OpsRamp REST API.
#
# msp.py
# Classes related to partner-level actions.
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import datetime
from opsramp.api import ORapi


class Clients(ORapi):
    def __init__(self, parent):
        super(Clients, self).__init__(parent.api, 'clients')

    def get(self, suffix='/minimal'):
        return self.api.get(suffix)

    def search(self, query_string=''):
        # For historical reasons the caller is allowed to omit the
        # queryString prefix, so add it if necessary.
        key = 'queryString='
        if query_string and key not in query_string:
            query_string = key + query_string
        return super(Clients, self).search(
            pattern=query_string
        )

    def create(self, definition):
        assert 'name' in definition
        assert 'address' in definition
        assert 'timeZone' in definition
        assert 'country' in definition
        return self.api.post('', json=definition)

    def update(self, uuid, definition):
        return self.api.post('%s' % uuid, json=definition)

    def activate(self, uuid):
        return self.api.post('%s/activate' % uuid)

    def suspend(self, uuid):
        return self.api.post('%s/suspend' % uuid)

    def terminate(self, uuid):
        return self.api.post('%s/terminate' % uuid)

    # Helper functions to create the complex structures that OpsRamp
    # uses to manipulate client definitions.
    @staticmethod
    def mkHours(day_start=None,
                day_end=None,
                week_start=2, week_end=6,
                sms_voice_notification=False):
        if not day_start:
            day_start = datetime.time(9, 0)
        if not day_end:
            day_end = datetime.time(17, 0)
        retval = {
            'businessStartHour': day_start.hour,
            'businessStartMin': day_start.minute,
            'businessEndHour': day_end.hour,
            'businessEndMin': day_end.minute,
            'businessDayStart': int(week_start),
            'businessDayEnd': int(week_end),
            'smsVoiceNotification': bool(sms_voice_notification)
        }
        return retval

    # A helper function to create the complex structures that OpsRamp
    # uses to define a new client. There are lots of optional fields and
    # potential gotchas here and we guard against *some* of them.
    @staticmethod
    def mkClient(name, address, time_zone, country,
                 hours=None):
        retval = {
            'name': name,
            'address': address,
            'timeZone': time_zone,
            'country': country
        }
        if hours:
            retval['clientDetails'] = hours
        # TODO there are lots and lots more optional fields that we
        # will probably need to cater for in the fullness of time.
        return retval



#!/usr/bin/env python
#
# Exercise the opsramp module as an illustration of how to use it.
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os
import yaml

import opsramp.binding
import opsramp.rba
import opsramp.msp


CATEGORY_NAME = 'python-opsramp test category'


def main():
    OPSRAMP_URL = os.environ['OPSRAMP_URL']
    TENANT_ID = os.environ['OPSRAMP_TENANT_ID']
    KEY = os.environ['OPSRAMP_KEY']
    SECRET = os.environ['OPSRAMP_SECRET']

    ormp = opsramp.binding.connect(OPSRAMP_URL, KEY, SECRET)

    # Print some random global "stuff" just to show that we can.
    cfg = ormp.config()
    print('alert types', cfg.get_alert_types())
    print('channels', cfg.get_channels())
    print('countries', len(cfg.get_countries()))
    print('timezones', len(cfg.get_timezones()))
    print('alert technologies', len(cfg.get_alert_technologies()))
    print('nocs', len(cfg.get_nocs()))
    print('device types', len(cfg.get_device_types()))

    # Focus on a specific tenant.
    tenant = ormp.tenant(TENANT_ID)

    if tenant.is_client():
        print('List the First Response Policies on tenant', TENANT_ID)
        group = tenant.first_response()
        found = group.search()
        print(found['totalResults'], 'first response policies')
        print(yaml.dump(found['results'], default_flow_style=False))

    if tenant.is_client():
        print('List the Service Maps on tenant', TENANT_ID)
        group = tenant.service_maps()
        found = group.get(minimal=True)
        print(len(found), 'service maps')
        print(yaml.dump(found, default_flow_style=False))

    print('List the RBAC permission sets on tenant', TENANT_ID)
    group = tenant.permission_sets()
    found = group.search()
    print(found['totalResults'], 'permission_sets')
    print(yaml.dump(found['results'], default_flow_style=False))

    print('List the RBAC roles on tenant', TENANT_ID)
    group = tenant.roles()
    found = group.search()
    print(found['totalResults'], 'roles')
    print(yaml.dump(found['results'], default_flow_style=False))

    print('List the Sites on tenant', TENANT_ID)
    group = tenant.sites()
    found = group.search()
    print(found['totalResults'], 'sites')
    print(yaml.dump(found['results'], default_flow_style=False))

    print('List the integrations on tenant', TENANT_ID)
    integs = tenant.integrations()
    group = integs.itypes()
    found = group.search()
    print(found['totalResults'], 'integration types')
    print(yaml.dump(found['results']))

    print('Define new custom integrations on', TENANT_ID)
    group = integs.instances()
    for atype in ('OAUTH2', 'BASIC'):
        newcint = group.mkCustom(
            display_name='Example %s integration' % atype,
            inbound_auth_type=atype
        )
        print(yaml.dump(newcint))
        # uncomment the following lines to actually create the integration.
        # if tenant.is_client():
        #     resp = group.create('CUSTOM', newcint)
        #     group.redact_response(resp)
        #     print(resp)

    found = group.search()
    print(found['totalResults'], 'integration instances')
    # "found" contains a complete description of each integration but let's
    # pull them individually anyway and assert that this gives same result.
    print('GET the integrations individually and compare')
    for i in found['results']:
        print('...', i['id'], i['integration']['id'],
              '"' + i.get('displayName', '<no name>') + '"')
        direct = group.get(i['id'])
        assert direct == i

    # Retrieve the agent installation script for this client. The string
    # that OpsRamp returns will contain keys for the client so just print
    # its length and first line to show that we got something.
    if tenant.is_client():
        print('agent installation script for client', TENANT_ID)
        agent_script = tenant.get_agent_script()
        print('length', len(agent_script))
        print(agent_script.split('\n')[0])
        print('')
    else:
        print('List the clients of tenant', TENANT_ID)
        group = tenant.clients()
        clist = group.get()
        print(len(clist), 'clients')
        print(yaml.dump(clist, width=150))
        for c in group.get():
            print('client', yaml.dump(c, width=150).strip())
            resp = group.get(c['uniqueId'])
            print('createdBy', yaml.dump(resp['createdBy'], width=150))
        print('Exercise the create-client code')
        cdef = group.mkClient(
            name='test client 99',
            address='Death Valley',
            time_zone='America/Los_Angeles',
            country='United States')
        print(yaml.dump(cdef))
        # uncomment these lines to actually create the client.
        # resp = group.create(cdef)
        # print(resp)

    print('List the Resources on tenant', TENANT_ID)
    if tenant.is_client():
        group = tenant.resources()
        found = group.minimal()
        print(found['totalResults'], 'resources')
        print(yaml.dump(found['results']))
    else:
        print('<clients only>')

    print('List the monitoring templates on tenant', TENANT_ID)
    monitoring = tenant.monitoring()
    templates = monitoring.templates()
    resp = templates.search()
    print(resp['totalResults'], 'templates')
    # There are typically hundreds of templates so don't YAML dump them.
    for t in resp['results']:
        print('...', t['name'])

    # list the RBA script categories and find or create the one we want.
    print('List the RBA categories on tenant', TENANT_ID)
    rba = tenant.rba()
    group = rba.categories()
    clist = group.get()
    print(len(clist), 'categories')
    print(yaml.dump(clist))
    for c in clist:
        if c['name'] == CATEGORY_NAME:
            cid = c['id']
            break
    else:
        # resp = group.create(CATEGORY_NAME)
        # cid = resp['id']
        print('Category {} not found'.format(CATEGORY_NAME))
        cid = None

    if cid:
        print('Scripts in category', cid)
        cobj = group.category(cid)
        slist = cobj.get()
        print(len(slist), 'scripts')
        for s in slist:
            print('...', s['id'], s['name'])
            direct = cobj.get(s['id'])
            assert s == direct

        # Create a new RBA script in this category with one parameter. You can
        # see from the "payload" that this gets passed to the script as $1
        p1 = cobj.mkParameter(
            name='venue',
            description='Where am I today?',
            datatype='STRING'
        )
        print('Parameter definition struct')
        print(yaml.dump(p1))
        s1 = cobj.mkScript(
            name='Hello <venue>',
            description='Stereotypical rock star intro',
            platforms=['LINUX'],
            execution_type='COMMAND',
            payload='echo "hello $1"',
            parameters=[p1]
        )
        print('Script definition struct')
        print(yaml.dump(s1))
        # uncomment these lines to actually create the script.
        # resp = cobj.create(s1)
        # print(resp)

    print('Management policies on tenant', TENANT_ID)
    group = tenant.policies()
    plist = group.get()
    print(len(plist), 'policies')
    for p in plist:
        rules = p['rules']
        actions = p['actions']
        print('...', p['id'], p['uid'], 'name', p['name'],
              'rules', len(rules),
              'actions', len(actions))
        for r in rules:
            print('...... rule', r)
        for a in actions:
            print('...... action', a['action'])
        # plist contains a complete description of each policy
        # but let's pull them individually anyway and assert that
        # this gives the same result.
        direct = group.get(p['id'])
        assert p == direct
    print('')

    print('Credential sets on tenant', TENANT_ID)
    if tenant.is_client():
        cs = tenant.credential_sets()
        resp = cs.get()
        print(resp['totalResults'], 'credential sets')
        print(yaml.dump(resp['results'], default_flow_style=False))
    else:
        print('<clients only>')

    print('Discovery Profiles on tenant', TENANT_ID)
    if tenant.is_client():
        group = tenant.discovery()
        profiles = group.get()
        print(len(profiles), 'profiles')
        for profile in profiles:
            uuid = profile['id']
            name = profile['name']
            print('...', uuid, '"%s"' % name)
            print(yaml.dump(profile))
            direct = group.get(uuid)
            assert direct == profile
        print('')
    else:
        print('<clients only>')

    print('Escalation Policies on tenant', TENANT_ID)
    group = tenant.escalations()
    found = group.search()
    print(found['totalResults'], 'escalation policies')
    print(yaml.dump(found['results'], default_flow_style=False))

    print('Management profiles on tenant', TENANT_ID)
    group = tenant.mgmt_profiles()
    found = group.search()
    print(found['totalResults'], 'management profiles')
    print(yaml.dump(found['results'], default_flow_style=False))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# A Python language binding for the OpsRamp REST API.
#
# integrations.py
# Classes related to Integrations.
#
# (c) Copyright 2019-2020 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import os

from opsramp.api import ORapi

'''
POST install/{intgld} e.g. CUSTOM

GET installed/search?queryString=
GET installed/{installedIntgld}
GET installed/{installedIntgld}/configFile/Kubernetes
DELETE installed/{installedIntgld}
POST installed/{installedIntgld}
POST installed/{installedIntgld}/notifier
POST installed/{installedIntgld}/enable
POST installed/{installedIntgld}/disable
POST installed/{installedIntgld}/inbound/authentication
POST installed/{installedIntgld}/mappingAttr

GET available/search?queryString=
GET available/{intgld}/emailProps/{entityType}
GET available/{intgld}/mappingAttr/{entityType}
'''


class Integrations(ORapi):
    def __init__(self, parent):
        super(Integrations, self).__init__(parent.api, 'integrations')

    def itypes(self):
        return Types(self)

    def instances(self):
        return Instances(self)

    def available(self):
        # Compatibility function because this is the name that the
        # OpsRamp API docs use for this data set.
        return self.itypes()

    def installed(self):
        # Compatibility function because this is the name that the
        # OpsRamp API docs use for this data set.
        return self.instances()


class Types(ORapi):
    def __init__(self, parent):
        super(Types, self).__init__(parent.api, 'available')


class Instances(ORapi):
    def __init__(self, parent):
        super(Instances, self).__init__(parent.api, 'installed')
        # The OpsRamp "create instance" API endpoint is in an
        # atypical place in the tree so need to compute it.
        self.creator_api = parent.api.clone()
        self.creator_api.chroot('install')

    def get_kubernetes_configuration(self, uuid):
        return self.api.get('%s/configFile/Kubernetes' % uuid)

    def create(self, type_name, definition):
        resp = self.creator_api.post(type_name, json=definition)
        return resp

    def delete(self, uuid, uninstall_reason="<Not specified>"):
        json_payload = {
            'uninstallReason': uninstall_reason
        }
        return self.api.delete(suffix=uuid, json=json_payload)

    def update(self, uuid, definition):
        return self.api.post('%s' % uuid, json=definition)

    def set_auth_type(self, uuid, auth_type):
        assert auth_type in ('OAUTH2', 'WEBHOOK', 'BASIC')
        settings = {
            'authType': auth_type
        }
        return self.api.post('%s/inbound/authentication' % uuid, json=settings)

    def enable(self, uuid):
        return self.api.post('%s/enable' % uuid)

    def disable(self, uuid):
        return self.api.post('%s/disable' % uuid)

    def notifier(self, uuid, definition):
        # Create or update installed integration base notifier
        return self.api.post('%s/notifier' % uuid, json=definition)

    # A helper function that extracts the authentication types from typical
    # integration instance response structs.
    @staticmethod
    def auth_type(resp):
        in_auth = resp.get(
            'inboundConfig', {}).get(
                'authentication', {}).get(
                    'authType', None)
        out_auth = resp.get(
            'outboundConfig', {}).get(
                'authentication', {}).get(
                    'authType', None)
        return (in_auth, out_auth)

    # A helper function that looks for login credentials in typical
    # integration instance response structs and redacts them.
    @staticmethod
    def redact_response(resp):
        for key in ('inboundConfig', 'outboundConfig'):
            config = resp.get(key, {})
            auth = config.get('authentication', {})
            if 'token' in auth:
                auth['token'] = 'REDACTED'
            for creds in auth.get('apiKeyPairs', {}):
                for i in creds.keys():
                    creds[i] = 'REDACTED'
        return resp

    # Helper functions to create the complex structures that OpsRamp
    # uses to define new integration instances. There are lots of
    # optional fields and potential gotchas here and we guard against
    # *some* of them.
    @staticmethod
    def mkBase(display_name,
               logo_fname=None):
        assert display_name
        retval = {
            'displayName': display_name,
        }
        if logo_fname:
            payload = ORapi.b64encode_payload(logo_fname)
            retval['logo'] = {
                'name': os.path.basename(logo_fname),
                'file': payload
            }
        return retval

    @staticmethod
    def mkEmailAlert(display_name,
                     logo_fname=None):
        assert display_name
        retval = Instances.mkBase(display_name, logo_fname)
        return retval

    @staticmethod
    def mkCustom(display_name,
                 logo_fname=None,
                 parent_uuid=None,
                 inbound_auth_type=None):
        assert display_name
        retval = Instances.mkBase(display_name, logo_fname)
        if parent_uuid:
            retval['parentIntg'] = {
                'id': parent_uuid
            }
        if inbound_auth_type:
            assert inbound_auth_type in ('OAUTH2', 'BASIC')
            retval['inboundConfig'] = {
                'authentication': {
                    'authType': inbound_auth_type
                }
            }
        return retval

    @staticmethod
    def mkAzureARM(display_name,
                   arm_subscription_id,
                   arm_tenant_id,
                   arm_client_id,
                   arm_secret_key):
        assert display_name
        retval = {
            'displayName': display_name,
            'credential': {
                'credentialType': 'AZURE',
                'AzureType': 'ARM',
                'SubscriptionId': arm_subscription_id,
                'TenantId': arm_tenant_id,
                'ClientID': arm_client_id,
                'SecretKey': arm_secret_key
            }
        }
        return retval

    @staticmethod
    def mkAzureASM(display_name,
                   arm_subscription_id,
                   arm_mgmt_cert,
                   arm_keystore_pass):
        assert display_name
        retval = {
            'displayName': display_name,
            'credential': {
                'credentialType': 'AZURE',
                'AzureType': 'ASM',
                'SubscriptionId': arm_subscription_id,
                'ManagementCertificate': arm_mgmt_cert,
                'KeystorePassword': arm_keystore_pass
            }
        }
        return retval

    @staticmethod
    def mkBaseNotifier(api_type, base_uri,
                       auth_type='OAUTH2',
                       grant_type='CLIENT_CREDENTIALS',
                       access_token_uri=None,
                       api_key=None,
                       api_secret=None,
                       user_name=None,
                       password=None):
        assert api_type in ('REST_API', 'SOAP_API')
        assert base_uri
        assert auth_type in ('NONE', 'BASIC', 'OAUTH2')
        retval = {
            'type': api_type,
            'baseURI': base_uri,
            'authType': auth_type
        }
        if auth_type != 'NONE':
            retval['grantType'] = grant_type
            if auth_type == 'OAUTH2':
                assert grant_type in ('CLIENT_CREDENTIALS', 'PASSWORD')
                # empty string is ok but omitting these isn't.
                assert access_token_uri is not None
                assert api_key is not None
                assert api_secret is not None
                retval['accessTokenURI'] = access_token_uri
                retval['apiKey'] = api_key
                retval['apiSecret'] = api_secret
            if grant_type == 'PASSWORD':
                # empty string is ok but omitting these isn't.
                assert user_name is not None
                assert password is not None
                retval['userName'] = user_name
                retval['password'] = password
        return retval

    # mkValueMap(('attrValue', 'tenantAttrValue'),
    #            (('client_113', '22cdbc5bb401d737b088c9'),
    #             ('client_843', '66cdbc5bb401d737b088c9'))
    @staticmethod
    def mkValueMap(labels, value_pairs):
        ormpname, othername = labels
        retval = [{ormpname: x[0], othername: x[1]} for x in value_pairs]
        return retval
